﻿using Correo.Middlewares;
using MediatR;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using NBB.Messaging.Host;

namespace Correo.Extensions
{
    public static class MessagingExtensions
    {
        public static void AddMessaging(this IServiceCollection services, IConfiguration configuration)
        {
            var enabled = configuration.GetValue<bool>("Messaging:Enabled", false);
            if (!enabled)
                return;

            services.AddMessageBus(configuration);
            services.AddMessagingHost(configuration);
        }

        private static void AddMessageBus(this IServiceCollection services, IConfiguration configuration)
        {
            services.AddMessageBus().AddNatsTransport(configuration);
        }

        private static void AddMessagingHost(this IServiceCollection services, IConfiguration configuration)
        {
            services.AddMessagingHost(configuration, hostBuilder =>
            {
                hostBuilder
                    .Configure(configBuilder => configBuilder
                        .AddSubscriberServices(config => config
                            .FromMediatRHandledCommands().AddClassesAssignableTo<IRequest>()
                            .FromMediatRHandledEvents().AddClassesAssignableTo<INotification>())
                        .WithDefaultOptions()
                        .UsePipeline(builder => builder
                            .UseCorrelationMiddleware()
                            .UseExceptionHandlingMiddleware()
                            .UseDefaultResiliencyMiddleware()
                            .UseMiddleware<SubscriberLoggingMiddleware>()
                            .UseMediatRMiddleware()));
            });
        }
    }
}
