using Correo.Extensions;
using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Serilog;
using System;

namespace Correo
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var builder = WebApplication.CreateBuilder(args);
            builder.Services.ConfigureServices(builder.Configuration);

            builder.Host.UseSerilog((_, loggerConfiguration) =>
            {
                loggerConfiguration
                .ReadFrom.Configuration(builder.Configuration)
                .Enrich.FromLogContext()
                .WriteTo.Console()
                .WriteToConfiguredDestinations(builder.Configuration);
            });

            var app = builder.Build();
            app.Configure();

            try
            {
                var urls = builder.Configuration.GetValue<string>("Urls");
                Log.Information("Starting Correo...");
                Log.Information($"Correo listening on {urls}");
                Console.WriteLine("Application started. Press Ctrl+C to shut down.");

                app.Run();
            }
            catch (Exception ex)
            {
                Log.Fatal(ex, "Correo host terminated unexpectedly");
            }
            finally
            {
                Log.CloseAndFlush();
            }
        }
    }
}